﻿#region

using System;
using MVR.FileManagementSecure;
using UnityEngine;
using UnityEngine.Serialization;

#endregion

namespace SceneNinja {
    [Serializable]
    public class PlayerData {
        [FormerlySerializedAs("Index")] [SerializeField]
        public int index;

        [FormerlySerializedAs("Id")] [SerializeField]
        public string id;

        [FormerlySerializedAs("Label")] [SerializeField]
        public string label;

        [FormerlySerializedAs("RightSide")] [SerializeField]
        public bool rightSide;

        [FormerlySerializedAs("AppearancePath")] [SerializeField]
        public string appearancePath;

        [FormerlySerializedAs("PluginsPath")] [SerializeField]
        public string pluginsPath;

        [FormerlySerializedAs("SuppressClothing")] [SerializeField]
        public bool suppressClothing;

        [FormerlySerializedAs("SuppressScale")] [SerializeField]
        public bool suppressScale;

        [FormerlySerializedAs("MergePlugins")] [SerializeField]
        public bool mergePlugins;

        [FormerlySerializedAs("IgnorePluginVersion")] [SerializeField]
        public bool ignorePluginVersion;


        private JSONStorableBool _suppressClothing;
        private JSONStorableBool _suppressScale;
        private JSONStorableBool _mergePlugins;
        private JSONStorableBool _ignorePluginVersion;

        private JSONStorableString _appearancePath;
        private JSONStorableString _pluginsPath;

        private JSONStorableString _appearanceText;
        private JSONStorableString _pluginsText;

        private SceneNinja _ninja;

        public PlayerData() { }

        public void LoadVars() {
            _suppressClothing = new JSONStorableBool(id + "SuppressClothing", suppressClothing, BoolUpdated);
            _suppressScale = new JSONStorableBool(id + "SuppressScale", suppressScale, BoolUpdated);
            _mergePlugins = new JSONStorableBool(id + "MergePlugins", mergePlugins, BoolUpdated);
            _ignorePluginVersion = new JSONStorableBool(id + "IgnorePluginVersion", ignorePluginVersion, BoolUpdated);
            _appearancePath = new JSONStorableString(id + "ApperancePath", appearancePath, StringUpdated);
            _pluginsPath = new JSONStorableString(id + "PluginsPath", pluginsPath, StringUpdated);
            _appearanceText = new JSONStorableString(id + "ApperanceText", "");
            _pluginsText = new JSONStorableString(id + "PluginsText", "");
            if (id.Contains("M")) {
                label = "Male " + id.Replace("M", "");
                rightSide = true;
            } else {
                label = "Female " + id.Replace("F", "");
                rightSide = false;
            }

            if (appearancePath != "") {
                var fileName = appearancePath;
                var parts = fileName.Split('/');
                var file = parts[parts.Length - 1];
                _appearanceText.val = file;
            }

            if (pluginsPath != "") {
                var fileName = pluginsPath;
                var parts = fileName.Split('/');
                var file = parts[parts.Length - 1];
                _pluginsText.val = file;
            }

        }

        public PlayerData(string id) {
            this.id = id;
            _suppressClothing = new JSONStorableBool(this.id + "SuppressClothing", true, BoolUpdated);
            _suppressScale = new JSONStorableBool(this.id + "SuppressScale", true, BoolUpdated);
            _mergePlugins = new JSONStorableBool(this.id + "MergePlugins", true, BoolUpdated);
            _ignorePluginVersion = new JSONStorableBool(this.id + "IgnorePluginVersion", true, BoolUpdated);
            _appearancePath = new JSONStorableString(this.id + "ApperancePath", "", StringUpdated);
            _pluginsPath = new JSONStorableString(this.id + "PluginsPath", "", StringUpdated);
            if (id.Contains("M")) {
                index = 1;
                if (id.Contains("2")) {
                    index = 3;
                }

                label = "Male " + id.Replace("M", "");
                rightSide = true;
            } else {
                index = 0;
                if (id.Contains("2")) {
                    index = 2;
                }

                label = "Female " + id.Replace("F", "");
                rightSide = false;
            }

            appearancePath = "";
            pluginsPath = "";
            _appearanceText = new JSONStorableString("AppearanceText", "");
            _pluginsText = new JSONStorableString("PluginsText", "");
            suppressClothing = true;
            suppressScale = true;
            mergePlugins = true;
        }

        private void StringUpdated(JSONStorableString js) {
            var name = js.name;
            if (name.Contains("Appearance")) {
                appearancePath = js.val;
            } else {
                pluginsPath = js.val;
            }

            _ninja.UpdatePlayer(this);
        }

        private void BoolUpdated(JSONStorableBool jb) {
            var name = jb.name;
            if (name.Contains("Clothing")) {
                suppressClothing = jb.val;
            }

            if (name.Contains("Scale")) {
                suppressScale = jb.val;
            }

            if (name.Contains("Merge")) {
                mergePlugins = jb.val;
            }

            if (name.Contains("Ignore")) {
                ignorePluginVersion = jb.val;
            }

            _ninja.UpdatePlayer(this);
        }

        public void Initialize(SceneNinja ninja) {
            _ninja = ninja;
            ninja.RegisterBool(_suppressClothing);
            ninja.RegisterBool(_suppressScale);
            ninja.RegisterBool(_mergePlugins);
            ninja.RegisterBool(_ignorePluginVersion);
            ninja.RegisterString(_appearancePath);
            ninja.RegisterString(_pluginsPath);
            ninja.CreateUILabel(label, rightSide);
            ninja.CreateUIButton(id + " Appearance", rightSide, SelectAppearanceFile);
            ninja.CreateUIButton("Clear Appearance", rightSide, ClearAppearanceFile);
            ninja.CreateUITextField(_appearanceText, rightSide, 40);
            ninja.CreateUIToggle(_suppressClothing, rightSide, "Suppress Clothing Load");
            ninja.CreateUIToggle(_suppressScale, rightSide, "Suppress Scale Load");
            ninja.CreateUIButton(id + " Plugins", rightSide, SelectPluginPreset);
            ninja.CreateUIButton("Clear Plugins", rightSide, ClearPluginsFile);
            ninja.CreateUITextField(_pluginsText, rightSide, 40);
            ninja.CreateUIToggle(_mergePlugins, rightSide, "Merge Load Plugins");
            ninja.CreateUIToggle(_ignorePluginVersion, rightSide, "Ignore Plugin Versions");
        }

        private void ClearPluginsFile() {
            PluginFileSelected("");
        }

        private void ClearAppearanceFile() {
            AppearanceFileSelected("");
        }

        private void SelectPluginPreset() {
            var shortcuts =
                FileManagerSecure.GetShortCutsForDirectory("Custom\\Atom\\Person\\Plugins", false, false, true);
            SuperController.singleton.GetMediaPathDialog(PluginFileSelected, "vap", "Custom\\Atom\\Person\\Plugins",
                false,
                true, false, null, false, shortcuts);
        }

        private void SelectAppearanceFile() {
            var shortcuts =
                FileManagerSecure.GetShortCutsForDirectory("Custom\\Atom\\Person\\Appearance", false, false, true);
            SuperController.singleton.GetMediaPathDialog(AppearanceFileSelected, "vap",
                "Custom\\Atom\\Person\\Appearance", false, true, false, null, false, shortcuts);
        }

        private void PluginFileSelected(string fileName) {
            var pFile = "";
            if (fileName != "" && !fileName.Contains("Appearance")) {
                pluginsPath = fileName;
                var parts = fileName.Split('/');
                pFile = parts[parts.Length - 1];
            }

            _pluginsPath.val = fileName;
            _pluginsText.val = pFile;
        }

        private void AppearanceFileSelected(string fileName) {
            var aFile = "";
            if (fileName != "" && !fileName.Contains("Plugins")) {
                appearancePath = fileName;
                var parts = fileName.Split('/');
                aFile = parts[parts.Length - 1];
            }

            _appearancePath.val = fileName;
            _appearanceText.val = aFile;
        }
    }
}
